# Summary
The mcPolymer simulator is published as open source software under the GNU General Public License (GPL). mcPolymer runs on different platforms: from workstation computers with Microsoft Windows up to compute servers running UNIX (Linux). By providing essentials of reaction templates, very different models of radical polymerization can be processed. Examples are simulations of reversible deactivation radical polymerizations (RAFT, ATRP, NMP), copolymerizations or a simulation of an acrylate polymerization with backbiting and transfer to polymer reactions. The sophisticated modular structure of mcPolymer allows the integration of the simulator in complex simulation environments, e.g. the parallelized simulation of emulsion polymerizations.

# Build environment
This section gives an introduction on how to build mcPolymer on a Linux system. However you can also use e.g. a Microsoft Visual Studio setup to build the software; there are no platform specific dependencies. You may also consider running a Windows subsystem for Linux setup (WSL2 e.g. Ubuntu 20.04 LTS) on a Windows Desktop.

For building mcPolymer with "make" on Linux, you will need:
* GCC version 7 and newer (we use some fancy C++17 features)
* Gtest/Gmock for the unit tests (tested with version 1.10)
* lcov for code coverage builds (tested with version 1.14)

On an Ubuntu system these packages are installed with this line:
```
> sudo apt-get install build-essential libgtest-dev lcov
```

# Build mcPolymer
The makefile provides several build targets. The default build provides the executable.
Further options are:
* make debug - builds mcPolymer with debug symbols; for debugging tools such as gdb
* make gtest - builds and executes unit tests
* make coverage - builds and executes unit test and creates a great looking html based code coverage
* make clean - removes all build artifacts

# Run mcPolymer
Show command line options:
```
> ./mcPolymer --help
```
Start example styrene.mcPolymer on command-line:
```
> ./mcPolymer -m Styrene.mcPolymer --time=3600 --temperature=70.0 --molecules=10000000000 --dt=60 --dtCLD=600
```

# Licence
mcPolymer - Monte Carlo simulation program of radical polymerization
Copyright (C) 2008 - 2020 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. 

# References
* M. Drache, K. Brandl, R. Reinhardt, S. Beuermann, Ab initio Kinetic Monte Carlo Simulation of Seeded Emulsion Polymerizations of Styrene, Phys. Chem. Chem. Phys. 2018, 20, 10796-10805.
* B. Hosemann, M. Drache, S. Beuermann, Advanced kinetic parameter fit applied to radical copolymerizations, Macromol. Symp. 2016, 370, 17-25.
* M. Drache, B. Hosemann, T. Laba, S. Beuermann, Modeling of Branching Distributions in Butyl Acrylate Polymerization Applying Monte Carlo Methods Macromol. Theory Simul.2015, 24, 301-310.
DOI: 10.1002/mats.201400081
* M. Drache, G. Drache, Simulating Controlled Radical Polymerizations with mcPolymer—A Monte Carlo Approach. Polymers2012, 4, 1416-1442.
