// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include <gtest/gtest.h>
#include "logging.h"

#include <filesystem>
#include <fstream>
#include <string>

using namespace mcPolymer_ns;

class LoggingTest : public ::testing::Test {
public:
	LoggingTest() {
	}
	~LoggingTest() {
	}

protected:

private:
	void SetUp() override {
		std::filesystem::path p(errorLogPath);
		if (std::filesystem::exists(std::filesystem::path(p)))
			std::filesystem::remove_all(p.parent_path());
	}
	void TearDown() override {
	}
};

TEST_F(LoggingTest, Logging)
{
	std::string line1 = "A log for the logfile";
	std::string line2 = "Function XX is not working";
	std::string line3 = "More logging";
	std::string line4 = "Baby does not want to sleep";

	EXPECT_NO_THROW(logError(line1));
	EXPECT_NO_THROW(logError(line2));
	EXPECT_NO_THROW(logError(line3));
	EXPECT_NO_THROW(logError(line4));
	EXPECT_TRUE(std::filesystem::exists(std::filesystem::path(errorLogPath)));

 	std::ifstream is(errorLogPath);
  	ASSERT_TRUE(is.is_open());
	std::string filecontent((std::istreambuf_iterator<char>(is)),
                       		(std::istreambuf_iterator<char>()));

	EXPECT_NE(filecontent.find(line1), std::string::npos);
	EXPECT_NE(filecontent.find(line2), std::string::npos);
	EXPECT_NE(filecontent.find(line3), std::string::npos);
	EXPECT_NE(filecontent.find(line4), std::string::npos);
}
