// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
 * Connect.h
 *
 *  Created on: 28.04.2016
 */

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <stdint.h>
#include "Connect.h"
#include "Fragment.h"

using namespace std;
namespace mcPolymer_ns {

// Only used during file import
Connect::Connect() : c1(nullptr), c2(nullptr), type(ConnectType::Undefined) {
}

Connect::Connect(Fragment *_c1, Fragment *_c2, ConnectType _type) {
	c1 = _c1;
	c2 = _c2;
	type = _type;
}

Connect::Connect(Connect *_Connect) {
	c1 = _Connect->c1;
	c2 = _Connect->c2;
	type = _Connect->type;
}

Connect::~Connect() {
}


Fragment* Connect::getConnect(int _number) const {
	if (_number == 1) {
		return c1;
	}
	if (_number == 2) {
		return c2;
	}
	return nullptr;
}

Fragment* Connect::getConnect12(const Fragment *_c) const {
	if (c1 == _c) {
		return c2;
	}
	if (c2 == _c) {
		return c1;
	}
	return nullptr;
}

Fragment* Connect::getConnect1() const {
	return c1;
}

Fragment* Connect::getConnect2() const {
	return c2;
}

bool Connect::isConnected(Fragment *_c) const {
	if ((c1 == _c) || (c2 == _c)) {
		return true;
	}
	return false;
}

ConnectType Connect::getType() const {
	return type;
}

void Connect::setFragment(int _number, Fragment *_c) {
	if (_number == 1) {
		c1 = _c;
	}
	if (_number == 2) {
		c2 = _c;
	}
}

void Connect::setType(ConnectType _type) {
	type = _type;
}
} // namespace
