// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
/*
 * class Fragment (chain)
 *
 *  Created on: 24.08.2008
 */

#pragma once

#include <valarray>
#include <iostream>
#include <fstream>
#include <stdint.h>
#include "globalTypes.h"

namespace mcPolymer_ns {

// defined by actual kinetic model at initialization - constant during runtime
// FIXME: find a good name for monomer types
extern size_t numberMonomers;

class Fragment {
public:
	// Only used during file import
	Fragment();

	Fragment(SpeciesID _id);
	Fragment(SpeciesID _id, uint32_t _length);
	Fragment(Fragment *_Fragment);
	Fragment(Fragment *_Fragment, double _randr);
	~Fragment();

	void exportFragment(std::ofstream* _os) const;
	void importFragment(std::ifstream* _is);

	const std::valarray<uint32_t> getMonomers() const;
	int getM(SpeciesID _id) const;
	size_t getLength() const;
	int getSCB() const;

	void addM(SpeciesID _id);
	void deleteM(SpeciesID _id);
	void deleteM(SpeciesID _id, uint32_t _count);
	void incrSCB();
	void deleteSCB(uint16_t _count);
	void addSCB(uint16_t _count);

private:
	uint16_t SCB;
	std::valarray<uint32_t> monomerCount;
};

} // namespace
