// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
/*
 * class Histogram
 *
 *  Created on: 7.11.2013
 */

#include <vector>
#include "Histogram.h"

using namespace std;

namespace mcPolymer_ns {

Histogram::Histogram() {
}

Histogram::~Histogram() {
	x.clear();
	y1.clear();
	y2.clear();
	y3.clear();
}
void Histogram::addValue(int _x, int _y1, int _y2, int _y3) {
	vector<int>::iterator xI, y1I, y2I, y3I;


	if (x.size() == 0) {
		x.resize(1);
		y1.resize(1);
		y2.resize(1);
		y3.resize(1);
		x.at(0) = _x;
		y1.at(0) = _y1;
		y2.at(0) = _y2;
		y3.at(0) = _y3;
		return;
	}

	xI = x.begin();
	y1I = y1.begin();
	y2I = y2.begin();
	y3I = y3.begin();

	for (size_t i = 0; i < x.size(); i++) {
		if (x.at(i) == _x) {
			y1.at(i) += _y1;
			y2.at(i) += _y2;
			y3.at(i) += _y3;
			return;
		}
		if (x.at(i) > _x) {
			x.insert(xI, _x);
			y1.insert(y1I, _y1);
			y2.insert(y2I, _y2);
			y3.insert(y3I, _y3);
			return;
		}
		++xI;
		++y1I;
		++y2I;
		++y3I;
	}
	x.push_back(_x);
	y1.push_back(_y1);
	y2.push_back(_y2);
	y3.push_back(_y3);
}
void Histogram::addValue(int _x, int _y1) {
	vector<int>::iterator xI, y1I;

	if (x.size() == 0) {
		x.resize(1);
		y1.resize(1);
		x.at(0) = _x;
		y1.at(0) = _y1;
		return;
	}

	xI = x.begin();
	y1I = y1.begin();

	for (size_t i = 0; i < x.size(); i++) {
		if (x.at(i) == _x) {
			y1.at(i) += _y1;
			return;
		}
		if (x.at(i) > _x) {
			x.insert(xI, _x);
			y1.insert(y1I, _y1);
			return;
		}
		++xI;
		++y1I;
	}
	x.push_back(_x);
	y1.push_back(_y1);
}

uint64_t Histogram::getSx() const {
	int sx = 0;
	for (size_t i = 0; i < x.size(); i++) {
		sx += x.at(i);
	}
	return sx;
}
uint64_t Histogram::getSy1() const {
	int sy = 0;
	for (size_t i = 0; i < y1.size(); i++) {
		sy += y1.at(i);
	}
	return sy;
}
uint64_t Histogram::getSy2() const {
	int sy = 0;
	for (size_t i = 0; i < y2.size(); i++) {
		sy += y2.at(i);
	}
	return sy;
}
uint64_t Histogram::getSy3() const {
	int sy = 0;
	for (size_t i = 0; i < y3.size(); i++) {
		sy += y3.at(i);
	}
	return sy;
}
int Histogram::getX(size_t _index) const {
	if (_index >= x.size()) {
		return 0;
	}
	return x[_index];
}
int Histogram::getY1(size_t _index) const {
	if (_index >= y1.size()) {
		return 0;
	}
	return y1[_index];
}
int Histogram::getY2(size_t _index) const {
	if (_index >= y2.size()) {
		return 0;
	}
	return y2[_index];
}
int Histogram::getY3(size_t _index) const {
	if (_index >= y3.size()) {
		return 0;
	}
	return y3[_index];
}
size_t Histogram::getSize() const {
	return x.size();
}
double Histogram::getPn() const {
	if (x.size() == 0) { return 0.0; }
	double snipi = 0.0;
	for (size_t i = 0; i < x.size(); i++) {
		snipi = snipi + 1.0 * x.at(i) * y1.at(i);
	}
	return snipi / getSy1();
}
double Histogram::getPw() const {
	if (x.size() == 0) { return 0.0; }
	double snipi = 0.0;
	double snipi2 = 0.0;

	for (size_t i = 0; i < x.size(); i++) {
		snipi = snipi + 1.0 * x.at(i) * y1.at(i);
		snipi2 = snipi2 + 1.0 * x.at(i) * x.at(i) * y1.at(i);
	}
	return snipi2 / snipi;
}

} // namespace
