// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
/*
 * class Histogram
 *
 *  Created on: 7.11.2013
 */

#pragma once
#include <cstddef>
#include <vector>
#include <cstdint>

namespace mcPolymer_ns {

class Histogram {
public:
	Histogram();
	~Histogram();
	void addValue(int _x, int _y1, int _y2, int _y3);
	void addValue(int _x, int _y1);
	uint64_t getSx() const;
	uint64_t getSy1() const;
	uint64_t getSy2() const;
	uint64_t getSy3() const;
	int getX(size_t _index) const;
	int getY1(size_t _index) const;
	int getY2(size_t _index) const;
	int getY3(size_t _index) const;
	size_t getSize() const;
	double getPn() const;
	double getPw() const;

private:
	std::vector<int> x;
	std::vector<int> y1;
	std::vector<int> y2;
	std::vector<int> y3;
};

} // namespace
