// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
/*
 * Plugin.h
 *
 *  Created on: 04.08.2017
 *
 */

#pragma once

#include <vector>
#include "PluginParameter.h"
#include "modelXML.h"
#include "Reactor.h"

namespace mcPolymer_ns {

class Plugin {
public:
	Plugin(int _id, const std::string& _outputDir);
	virtual ~Plugin() = default;
	int getID();
	std::string getOutputDir();
private:
	int id;
	std::string outputDir;
};

class TerminationHamielec : public Plugin {
public:
	TerminationHamielec(int _id, const std::string& _outputDir);
	void setCountStyrene0(uint64_t _count);
	uint64_t getCountStyrene0();
	double getScaleFactor(double _temperature, double _conversion);

private:
	parameterPluginTerminationHamielec parameter;
	uint64_t countStyrene0 = 0;
};

class TerminationChainLengthDependent : public Plugin {
public:
	TerminationChainLengthDependent(int _id, const std::string& _outputDir);
	void setParameter(int _ic, double _as, double _al, double _as2, double _as3, double _as4, double _kt11p1, double _kt11p2, double _kt11p3);
	double calcFactorCliClj(const std::vector<int> &_cli, const std::vector<int> &_clj);
private:
	parameterPluginTerminationChainLengthDependent parameter;
	int coefficientID = -1;
	std::vector<size_t> terminationReactionIndex;
};
} // namespace

