// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
/*
*
*  Created on: 28.03.2018
*/
#pragma once
namespace mcPolymer_ns {


// M. Drache, G. Drache, Simulating Controlled Radical Polymerizations with 
// mcPolymer � A Monte Carlo Approach, Polymers 2012, 4, 1416-1442.
// Hui, A.W.; Hamielec, A.E.Thermal polymerization of styrene at high conversions and
// temperatures.An Experimental study.J.Appl.Polym.Sci. 1972, 16, 749�769.
struct parameterPluginTerminationHamielec {
	// runlevel:
	// execution before each call to runSimulationUntil

	double a10 = 2.57;
	double a11 = -5.05E-3;
	double a20 = 9.56;
	double a21 = -1.76E-2;
	double a30 = -3.03;
	double a31 = 7.85E-3;
};

// M. Drache, B. Hosemann, T. Laba, S. Beuermann, Modeling of Branching Distributions in Butyl Acrylate 
// Polymerization Applying Monte Carlo Methods, Macromol. Theory Simul. 2015, 24, 301-310.
// J. Barth, M. Buback, P. Hesse and T. Sergeeva
// Termination and Transfer Kinetics of Butyl Acrylate Radical Polymerization Studied via SP-PLP-EPR
// Macromolecules, 2010, 43, 4023�4031
struct parameterPluginTerminationChainLengthDependent {

	// runlevel:
	// execution before each call to runSimulationUntil
	
	// ID: undefined by default
	int ktID = -1;
	
	double kt11A0 = 1.3e10;
	double kt11Ea = 8.4;
	
	double al = 0.22;
	double as = 0.85;
	double ic = 30;
	double as2 = 0.0;
	double as3 = 0.0;
	double as4 = 0.0;
	double kt11p1 = 0.0;
	double kt11p2 = 0.0;
	double kt11p3 = 0.0;
	double kt11Offset = 1.0;
	bool polymerContentDependent = false;
};

// M. Drache, B. Hosemann, T. Laba, S. Beuermann, Modeling of Branching Distributions in Butyl Acrylate 
// Polymerization Applying Monte Carlo Methods, Macromol. Theory Simul. 2015, 24, 301-310.
// J. Barth, M. Buback, P. Hesse and T. Sergeeva
// Termination and Transfer Kinetics of Butyl Acrylate Radical Polymerization Studied via SP-PLP-EPR
// Macromolecules, 2010, 43, 4023�4031
struct parameterPluginCrossTerminationChainLengthDependent {

	// runlevel:
	// execution before each call to runSimulationUntil
	
	// ID: undefined by default
	int ktID = -1;
	
	double kt11A0 = 4.2e9;
	double kt11Ea = 6.6;

	double al = 0.22;
	double as = 0.85;
	double ic = 30;
};

} // namespace
