// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
/*
 * class Polymer
 *
 *  Created on: 24.08.2008
 */

#pragma once

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <math.h>
#include <stdint.h>
#include "modelObjects.h"
#include "Histogram.h"
#include "Fragment.h"
#include "Connect.h"

namespace mcPolymer_ns {

class Polymer {
public:
	Polymer();

	// FIXME: Calling the following two constructors caused a core dump on default conditions :(
	// Seems to be the case if mcPolymer_ns::numberMonomers == 0
	Polymer(int id);
	Polymer(int id, int _length);
	Polymer(Fragment* _Fragment);
	~Polymer();
	int getFragmentSize();
	bool exportPolymer(std::ofstream* _os);
	bool importPolymer(std::ifstream* _is);
	void select();
	// FIXME: name unselect?
	void deselect();
	bool getSelection();
	const std::valarray<uint32_t> getMonomers();
	void addM1SCB(int _id);
	// FIXME: get number of connects?
	int getConnects();
	Connect* getConnect(size_t _index);
	Connect* deleteConnect();
	void deleteLastConnect();
	Fragment* getConnect(size_t _index, int _number);
	ConnectType getConnectType(size_t _index);
	bool setConnectType(size_t _index, ConnectType _type);
	void addConnect(Fragment* _frg1, Fragment* _frg2, ConnectType _type);
	Polymer* splitPolymerAtConnectType2();
	void addConnect(Connect* _c);
	int getM(int id);
	void activateFragment(int _idS, int _idD);
	Fragment* getFragment(int ind);
	Fragment* getLastFragment();
	void addFragment(Fragment* c1);
	// FIXME: popFragment better name?
	Fragment* deleteFragment();
	int getLength();
	int getSCB();
	size_t getNumLCB();
	void activateECR();
	void activateECR(Fragment* _activeFragment);
	void removeRadical();
	Fragment* getPropagatingFragment();
	PropagatingRadicalType getPropagatingRadicalType();
	void backbiting(uint32_t radicalPosition);
	void randomizeRadicalPosition();
	Polymer* fragmentation();
	void activateMCR(int _eductID, int _productID, PropagatingRadicalType _RadicalType);
	uint32_t getNumTransferMonomers();
	void addM(int _id);
	void addSCB();

private:
	struct environmentMCR {
		// Position relative to chain end (radical type 2)
		uint32_t getShortFragmentSize() {
			uint32_t count = 0;
			for (size_t i = 0; i < SCB.size(); i++) {
				if (SCB.at(i)) {
					++count;
				}
			}
			return (radicalPosition + count);
		}
		void blockCurrentPosition() {
			if (radicalPosition < SCB.size()) {
				bool blocked = true;
				SCB.at(radicalPosition) = blocked;
			}
		}
		bool blockedPositionPlus() {
			bool blocked = false;
			if (radicalPosition + 1 < SCB.size()) {
				if (SCB.at(radicalPosition + 1)) {
					blocked = true;
				}
			}
			return blocked;
		}
		bool SCBPosition() {
			return SCB.at(radicalPosition);
		}
		void setNewRadicalPosition(uint32_t _Position) {
			radicalPosition = _Position;
			if (_Position < SCB.size()) {
				return;
			}
			else {
				radicalPosition = _Position;
				size_t addPosSCB = radicalPosition - SCB.size() + 1;
				for (size_t i = 0; i < addPosSCB; i++) {
					bool flagSCB = false;
					SCB.push_back(flagSCB);
				}
				return;
			}
		}
		uint16_t getCountSCB() {
			uint16_t count = 0;
			for (size_t i = 0; i < SCB.size(); i++) {
				if (SCB.at(i)) {
					++count;
				}
			}
			return count;
		}
		uint32_t radicalPosition = 0;
		std::vector<bool> SCB;
	};
	struct Radical {
		Fragment* activeFragment = nullptr;
		Connect* activeConnect = nullptr;
		environmentMCR* MCRS = nullptr;
		PropagatingRadicalType radicalType = PropagatingRadicalType::Undefined;
	};
	std::vector<Connect*> fragmentConnects;
	std::vector<Fragment*> polymerFragment;
	Radical propagatingRadical;
	bool selectPolymer;
	int idTransferMonomer = 0;
	size_t getFragmentIndex(Fragment* _c);
};

} // namespace
