// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
/*
 *  Created on: 01.03.2021
 */

#pragma once

#include "MersenneTwister.h"
#include <string>
#include <vector>
#include <stdint.h>

namespace mcPolymer_ns {

class RandomBuffer {
public:
    RandomBuffer(uint16_t _sizeMBytes, uint16_t partitioning = 2);
    double getRand();
    bool fillRands(uint32_t _oneseed = UINT32_MAX);
    void setExit();

    const double REFILL = 2.0;
    const double EXIT = 3.0;
    const double READY = 4.0;

private:
    std::vector<double> buffer;
    std::vector<double>::const_iterator nextPos;
    uint16_t partitioning;
};

// FIXME: Make singleton
extern RandomBuffer *randBufferInstance;


} // namespace
