// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
* Created on: 13.02.2021
*/
#pragma once

#include "cmdlineOptions.h"
#include "simulation.h"
#include "modelInterpreter.h"

#include <iostream>
#include <fstream>
#include <vector>

namespace mcPolymer_ns {

class SimulationController;

void prepareSimulation(int argc, char* argv[]);
void startSimulation(SimulationController &simController);
void startRandomThread();
void setupThreads(SimulationController &simController);

class SimulationController {
public:
	SimulationController(const mcPolymerParameter &_simulationParameter);
	~SimulationController();
	void runSimulation();
protected:
	void exportComplete(time_t _timerStart) const;
	void exportResults(double _reactionTime, bool _initial) const;
	void exportPolymers(double _reactionTime) const;
	void initCurrentModel();
private:
	const mcPolymerParameter simulationParameter;
	Simulation *kMC;
	modelXML currentModel;
};

} // namespace
