// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
 * binaryTree.h
 *
 * Created on: 23.09.2020
 */

#pragma once
#include <vector>
#include "Polymer.h"
#include "MersenneTwister.h"

namespace mcPolymer_ns {

class Node {
public:
	Node(Node* _father, Node* _leftChild, Node* _rightChild);
	void connectFather(Node* _father);
	void connectChilds(Node* _leftChild, Node* _rightChild);
	void addMonomers(uint64_t _monomers);
	void delMonomers(uint64_t _monomers);
	Node* getFather() const;
	Node* getLeftChild() const;
	Node* getrightChild() const;
	uint64_t getSumMonomers() const;
private:
	uint64_t sumMonomers = 0;
	Node* father;
	Node* leftChild;
	Node* rightChild;
};

class lastLevelNode : public Node {
public:
	lastLevelNode(Node* _father, Node* _leftChild, Node* _rightChild);
	void addPolymer(Polymer* _polymer);
	uint64_t getNumPolymers() const;
	Polymer* getPolymer(size_t _indexPolymer);
	void deletePolymer(Polymer* _polymer);
	std::vector<Polymer*> analyzeAllPolymers() const;
private:
	std::vector<Polymer*> Polymers;
};

class binTree {
public:
	binTree(int _bits);
	void addPolymer(Polymer* _polymer);
	Polymer* getPolymer(uint64_t _numMonomer) const;
	Polymer* getPolymerLastLevel(uint64_t _targetPolymer) const;
	void deletePolymer(Polymer* _polymer) const;
	size_t getNumPolymers() const;
	uint64_t getSumMonomers() const;
	std::vector<Polymer*> analyzeAllPolymers() const;
	void exportPolymers(std::ofstream* _os) const;
private:
	Node* root;
	std::vector<Node*> allNodes;
	std::vector<lastLevelNode*> lastLevelNodes;
	mutable MTRand mtrandr;
};

} // namespace
