// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
/*
 *  Created on: 08.02.2021
 */

#include <chrono>
#include <ctime>
#include <iostream>
#include <filesystem>
#include <fstream>
#include <sstream>
#include <string>

#include "logging.h"

namespace mcPolymer_ns {

void createLogfilePath() {
	namespace fs = std::filesystem;
	fs::path path(errorLogPath);
	fs::create_directories(path.parent_path());
}

std::string getTimestampString() {
	auto now = std::chrono::system_clock::now();
	auto epoch_seconds = std::chrono::system_clock::to_time_t(now);

	std::stringstream ss;
	// ??! set _CRT_SECURE_NO_WARNINGS
	ss << std::put_time(localtime(&epoch_seconds), "%FT%T");
	auto truncated = std::chrono::system_clock::from_time_t(epoch_seconds);
	auto delta_us = std::chrono::duration_cast<std::chrono::microseconds>(now - truncated).count();
	ss << "." << std::fixed << std::setw(6) << std::setfill('0') << delta_us << " ";
	return ss.str();
}

void logError(const std::string& _line) {
	namespace fs = std::filesystem;
	fs::path path(errorLogPath);
	if (!fs::exists(path))
		createLogfilePath();
	std::ofstream os(errorLogPath, std::ios::out | std::ios::app);
	if (!os.is_open())
		return;
	os << getTimestampString() << _line << std::endl;
}

} // namespace
