// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
* modelObjects.cpp
*
*  Created on: 10.02.2021
*/
#include <string>
#include "modelObjects.h"
namespace mcPolymer_ns {

std::string PluginTypeToString(PluginType _type) {
	switch (_type) {
	case PluginType::TerminationHamielec:
		return "TerminationHamielec";
	case PluginType::TerminationChainLengthDependent:
		return "TerminationChainLengthDependent";
	default:
		return "Undefined";
	}
}
PluginType StringToPluginType(const std::string &_str) {
	if (_str == "TerminationHamielec") {
		return PluginType::TerminationHamielec;
	}
	if (_str == "TerminationChainLengthDependent") {
		return PluginType::TerminationChainLengthDependent;
	}
	return PluginType::Undefined;
}
std::string ReactionTemplateToString(ReactionTemplate _template) {
	switch (_template) {
	case ReactionTemplate::Backbiting_RadPos3:
		return "Backbiting_RadPos3";
	case ReactionTemplate::ElementaryReaction:
		return "ElementaryReaction";
	case ReactionTemplate::Fragmentation:
		return "Fragmentation";
	case ReactionTemplate::Initiation:
		return "Initiation";
	case ReactionTemplate::InitiatorDecomposition:
		return "InitiatorDecomposition";
	case ReactionTemplate::Propagation_PnPnp1:
		return "Propagation_PnPnp1";
	case ReactionTemplate::Propagation_PnQnp1:
		return "Propagation_PnQnp1";
	case ReactionTemplate::Propagation_SCB:
		return "Propagation_SCB";
	case ReactionTemplate::Termination_PnPmDnDm:
		return "Termination_PnPmDnDm";
	case ReactionTemplate::Termination_PnPmDnpm:
		return "Termination_PnPmDnpm";
	case ReactionTemplate::ThermalInitiation3:
		return "ThermalInitiation3";
	case ReactionTemplate::Transfer:
		return "Transfer";
	case ReactionTemplate::TransferToMonomer:
		return "TransferToMonomer";
	case ReactionTemplate::TransferToPolymer:
		return "TransferToPolymer";
	default:
		return "Undefined";
	}
}
ReactionTemplate StringToReactionTemplate(const std::string& _str) {
	if (_str == "Backbiting_RadPos3") {
		return ReactionTemplate::Backbiting_RadPos3;
	}
	if (_str == "ElementaryReaction") {
		return ReactionTemplate::ElementaryReaction;
	}
	if (_str == "Fragmentation") {
		return ReactionTemplate::Fragmentation;
	}
	if (_str == "Initiation") {
		return ReactionTemplate::Initiation;
	}
	if (_str == "InitiatorDecomposition") {
		return ReactionTemplate::InitiatorDecomposition;
	}
	if (_str == "Propagation_PnPnp1") {
		return ReactionTemplate::Propagation_PnPnp1;
	}
	if (_str == "Propagation_PnQnp1") {
		return ReactionTemplate::Propagation_PnQnp1;
	}
	if (_str == "Propagation_SCB") {
		return ReactionTemplate::Propagation_SCB;
	}
	if (_str == "Termination_PnPmDnDm") {
		return ReactionTemplate::Termination_PnPmDnDm;
	}
	if (_str == "Termination_PnPmDnpm") {
		return ReactionTemplate::Termination_PnPmDnpm;
	}
	if (_str == "ThermalInitiation3") {
		return ReactionTemplate::ThermalInitiation3;
	}
	if (_str == "Transfer") {
		return ReactionTemplate::Transfer;
	}
	if (_str == "TransferToMonomer") {
		return ReactionTemplate::TransferToMonomer;
	}
	if (_str == "TransferToPolymer") {
		return ReactionTemplate::TransferToPolymer;
	}
	return ReactionTemplate::Undefined;
}
}