// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
* modelObjects.h
*
*  Created on: 07.02.2021
*/
#include <string>
#include <vector>
#pragma once
namespace mcPolymer_ns {
enum class PluginType : uint16_t {
	TerminationHamielec = 0,
	TerminationChainLengthDependent,
	Undefined = 99
};
std::string PluginTypeToString(PluginType _type);
PluginType StringToPluginType(const std::string &_str);

enum class ReactionTemplate : uint16_t {
	Backbiting_RadPos3 = 0,
	ElementaryReaction,
	Fragmentation,
	Initiation,
	InitiatorDecomposition,
	Propagation_PnPnp1,
	Propagation_PnQnp1,
	Propagation_SCB,
	Termination_PnPmDnDm,
	Termination_PnPmDnpm,
	ThermalInitiation3,
	Transfer,
	TransferToMonomer,
	TransferToPolymer,
	Undefined = 99
};
std::string ReactionTemplateToString(ReactionTemplate _rktTemplate);
ReactionTemplate StringToReactionTemplate(const std::string& _str);

enum class PropagatingRadicalType : uint16_t {
	ECR = 0,
	MCR,
	MCRL,
	Undefined = 99
};

enum class ConnectType : uint16_t {
	Termination = 0,
	LCB,
	Crosslink,
	Undefined = 99
};

enum class FileTypePolymerSpecies : uint16_t {
	CLD = 0,
	Binary,
	Undefined = 99
};

enum class VolumeBalanceMode {
	Internal,
	External,
	None
};

struct polymerSpeciesProperties {
	size_t numPolymers = 0;
	std::vector<uint64_t> numMonomers;
	uint64_t numSCB = 0;
	uint64_t numLCB = 0;
};
struct chainLengthDependence {
	int ic = 0;
	double as = 0.0;
	double al = 0.0;
	double as2 = 0.0;
	double as3 = 0.0;
	double as4 = 0.0;
	double kt11p1 = 0.0;
	double kt11p2 = 0.0;
	double kt11p3 = 0.0;
};
struct speciesObject {
	int id = 0;
	std::string name = "";
	bool isInitiator = false;
	bool isMonomer = false;
	bool isPolymer = false;
	double concentration = 0.0;
	double optionalParameterMolarMass = 0.0;
	double optionalParameterF = 0.0;
	uint64_t molecules = 0;
};
struct coefficientObject {
	int id = 0;
	std::string name = "";
	bool isTemperatureDependent = false;
	bool isPressureDependent = false;
	double A0 = 0.0;
	double Ea = 0.0;
	double deltaV = 0.0;
	double value = 0.0;
	bool initialized = false;
	bool order2EE = false;
	uint64_t reactionOrder;
	double valueMC;
};
struct reactionObject {
	int id = 0;
	ReactionTemplate rktTemplate = ReactionTemplate::Undefined;
	int coefficientID = 0;
	std::vector<int> eductIdList;
	std::vector<int> productIDList;
};
struct pluginObject {
	int id = 0;
	PluginType type = PluginType::Undefined;
	std::string templateLabel = "";
	std::string templateLabelFilename = "";
	std::string filename = "";
	int coefficientID = -1;
	std::vector<int> speciesIdList;
	chainLengthDependence chainLengthDependenceParameter;
};
struct reactorObject {
	double controlVolume = 0.0;
	double t0controlVolume = 0.0;
	double volume = 1.0;
	double t0Volume = 1.0;
	uint64_t molecules;
	double temperature = -273.15;
	double pressure = 1.0;
};
} // namespace
