// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
* modelXML.cpp
*
* Created on: 15.02.2020
*/
#pragma once

#include <string>
#include <vector>
#include <map>
#include "modelObjects.h"
namespace mcPolymer_ns {

class modelXML {
public:
	modelXML();
	~modelXML();

	bool constructModel(std::vector<speciesObject> _speciesObjects,
						std::vector<coefficientObject> _coefficientObjects,
						std::vector<reactionObject> _reactionObjects,
						std::vector<pluginObject> _pluginObjects);

	void initModel(double _controlVolume, double _temperature, double _pressure);
	void initModel(uint64_t _molecules, double _temperature, double _pressure);
	void scaleControlVolume(double _factor);
	void setControlVolume(double _controlVolume);
	void setTemperature(double _temperature);
	const std::vector<speciesObject> getSpecies() const;
	const std::vector<coefficientObject> calculateAndGetCoefficients();
	const std::vector<reactionObject> getReactions() const;
	const std::vector<pluginObject> getPlugins() const;
	double getRatioConcMolecules() const;
	double getVolumet0();
	double getControlVolume();
	double getVolume();
	double getTemperature();

protected:
	void addSpeciesObjects(std::vector<speciesObject>& _speciesObjects);
	void addCoefficientObjects(std::vector<coefficientObject>& _coefficientObjects);
	bool addReactionObjects(std::vector<reactionObject>& _reactionObjects);
	void addPluginObjects(std::vector<pluginObject>& _pluginObjects);

private:
	std::vector<speciesObject> speciesObjectList;
	std::vector<coefficientObject> coefficientObjectList;
	std::vector<reactionObject> reactionObjectList;
	std::vector<pluginObject> pluginObjectList;
	void initCoefficients();
	void calcultateCoefficients();
	size_t currentIndexProfiles = 1;
	const double AvogadroConstantNA = 6.022140857e23;
	reactorObject reactor;
};

} // namespace
