// mcPolymer - Monte Carlo simulation program of radical polymerization
// Copyright (C) 2008 - 2021 Marco Drache, Georg Drache, Benjamin Hosemann, Katrin Brandl
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

/*
* class polymerSpecies
*
*  Created on: 16.01.2014
*/

#pragma once

#include <stdio.h>
#include <map>
#include "modelXML.h"
#include "Polymer.h"
#include "binaryTree.h"
#include "MersenneTwister.h"
namespace mcPolymer_ns {

class polymerSpecies {
public:
	polymerSpecies(int _id);
	~polymerSpecies();
	polymerSpeciesProperties getPolymerSpeciesProperties();
	bool importSpecies(const std::string &_filename);
	void exportCLD(const std::string& _filename, const modelXML* _mcPolymerModel);
	void writeCLD(const std::string& _filename, const std::map<int, std::string>& _speciesNameMap, const std::map<int, int>& _speciesTypeMap, const std::map<int, double>& _molarMassMap) const;
	void exportPolymers(std::ofstream* _os) const;
	void getCLD(Histogram* _CLD, const std::vector<Histogram*>& _monomerHistogram);
	std::vector<int> getChainLengthList() const;
	size_t getSize() const;
	int getLength(int _index) const;
	int getID() const;
	void setTransferMonomer(int _id);
	uint64_t getNumMonomer(int _id) const;
	void newP1(int _id);
	void incrP(int _id);
	Polymer* deleteP(uint64_t _numPolymers);
	Polymer* deleteP();
	Polymer* deletePolymerForTransfer(int _id);
	uint64_t getNumTransferMonomers() const;
	void addPolymer(Polymer* p1);
	void addPolymers(std::vector<Polymer*> _p);
	void addPolymerFromCombination(Polymer* p1, Polymer* p2);
	std::vector<Polymer*> getAllPolymers();
private:
	Polymer* getPolymer() const;
	void exportSelectedPolymers(uint64_t sizeSelected) const;
	std::vector<Polymer*> p;
	mutable MTRand mtrandr;
	int id;
	bool transferToPolymer = false;
	int transferMonomerID = 0;
	binTree* bintree;
	polymerSpeciesProperties properties;
};

} // namespace
